@@ -19,24 +19,95 @@
 import hashlib
 import logging
 import os
+from collections import defaultdict
 from collections.abc import Mapping
 from typing import Any, Dict, List, Optional, Union
 
+import numpy as np
 import pyarrow as pa
 from tqdm import tqdm
 
 from nlp.utils.py_utils import dumps
 
 from .arrow_writer import ArrowWriter
-from .utils import convert_tuples_in_lists
+from .utils import convert_tuples_in_lists, map_nested
 
 
 logger = logging.getLogger(__name__)
 
 
-class Dataset(object):
-    """ A Dataset backed by an Arrow table or Record Batch
-        This allow us to do memory mapping to disk i.e. train on datasets larger than RAM
+class DatasetInfoMixin(object):
+    """ This base class exposes some attributes of DatasetInfo
+        at the base level of the Dataset for easy access.
+    """
+
+    def __init__(self, info, split):
+        self._info = info
+        self._split = split
+
+    @property
+    def info(self):
+        return self._info
+
+    @property
+    def split(self):
+        return self._split
+
+    @property
+    def builder_name(self) -> str:
+        return self._info.builder_name
+
+    @property
+    def citation(self) -> str:
+        return self._info.citation
+
+    @property
+    def config_name(self) -> str:
+        return self._info.config_name
+
+    @property
+    def dataset_size(self) -> Optional[int]:
+        return self._info.dataset_size
+
+    @property
+    def description(self) -> str:
+        return self._info.description
+
+    @property
+    def download_checksums(self) -> Optional[dict]:
+        return self._info.download_checksums
+
+    @property
+    def download_size(self) -> Optional[int]:
+        return self._info.download_size
+
+    @property
+    def features(self):
+        return self._info.features
+
+    @property
+    def homepage(self) -> Optional[str]:
+        return self._info.homepage
+
+    @property
+    def license(self) -> Optional[str]:
+        return self._info.license
+
+    @property
+    def size_in_bytes(self) -> Optional[int]:
+        return self._info.size_in_bytes
+
+    @property
+    def supervised_keys(self):
+        return self._info.supervised_keys
+
+    @property
+    def version(self):
+        return self._info.version
+
+
+class Dataset(DatasetInfoMixin):
+    """ A Dataset backed by an Arrow table or Record Batch.
     """
 
     def __init__(
@@ -44,30 +115,30 @@ class Dataset(object):
         arrow_table: Union[pa.Table, pa.RecordBatch],
         data_files: Optional[List[dict]] = None,
         info: Optional[Any] = None,
+        split: Optional[Any] = None,
     ):
-        self._info = info
+        super().__init__(info=info, split=split)
         self._data: pa.Table = arrow_table
         self._data_files: List[dict] = data_files if data_files is not None else []
         self._format_type = None
         self._format_columns = None
+        self._output_all_columns = False
 
     @classmethod
-    def from_file(cls, filename):
+    def from_file(cls, filename: str, info: Optional[Any] = None, split: Optional[Any] = None):
+        """ Instantiate a Dataset backed by an Arrow table at filename """
         mmap = pa.memory_map(filename)
         f = pa.ipc.open_stream(mmap)
         pa_table = f.read_all()
-        return cls(arrow_table=pa_table, data_files=[{"filename": filename}])
+        return cls(arrow_table=pa_table, data_files=[{"filename": filename}], info=info, split=split)
 
     @classmethod
-    def from_buffer(cls, buffer):
+    def from_buffer(cls, buffer: pa.Buffer, info: Optional[Any] = None, split: Optional[Any] = None):
+        """ Instantiate a Dataset backed by an Arrow buffer """
         mmap = pa.BufferReader(buffer)
         f = pa.ipc.open_stream(mmap)
         pa_table = f.read_all()
-        return cls(pa_table)
-
-    @property
-    def info(self):
-        return self._info
+        return cls(pa_table, info=info, split=split)
 
     @property
     def data(self):
@@ -159,8 +230,13 @@ class Dataset(object):
         return self._data.num_rows
 
     def __iter__(self):
+        format_type = self._format_type
+        format_columns = self._format_columns
+        output_all_columns = self._output_all_columns
         for index in range(self._data.num_rows):
-            yield self.unnest(self._data.slice(index, 1).to_pydict())
+            yield self._getitem(
+                index, format_type=format_type, format_columns=format_columns, output_all_columns=output_all_columns,
+            )
 
     def __repr__(self):
         schema_str = dict((a, str(b)) for a, b in zip(self._data.schema.names, self._data.schema.types))
@@ -171,9 +247,19 @@ class Dataset(object):
         return {
             "type": "python" if self._format_type is None else self._format_type,
             "columns": self.column_names if self._format_columns is None else self._format_columns,
+            "output_all_columns": self._output_all_columns,
         }
 
-    def set_format(self, type: Optional[str] = None, columns: Optional[List] = None):
+    def set_format(self, type: Optional[str] = None, columns: Optional[List] = None, output_all_columns: bool = False):
+        """ Set __getitem__ return format (type and columns)
+
+            Args:
+                type (Optional ``str``): output type selected in [None, 'numpy', 'torch', 'tensorflow', 'pandas']
+                    None means __getitem__ returns python objects (default)
+                columns (Optional ``List[str]``): columns to format in the output
+                    None means __getitem__ returns all columns (default)
+                output_all_columns (``bool`` default to False): keep un-formated columns as well in the output (as python objects)
+        """
         # Check return type
         if type == "torch":
             try:
@@ -202,30 +288,39 @@ class Dataset(object):
 
         self._format_type = type
         self._format_columns = columns
+        self._output_all_columns = output_all_columns
         logger.info(
-            "Set __getitem__(key) output type to %s and filter %s columns " " (when key is int or slice).",
+            "Set __getitem__(key) output type to %s for %s columns "
+            " (when key is int or slice) and %s output other (un-formated) columns.",
             "python objects" if type is None else type,
             "no" if columns is None else str(columns),
+            "do" if output_all_columns else "don't",
         )
 
     def reset_format(self):
+        """ Reset __getitem__ return format to python objects and all columns.
+
+            Same as ``self.set_format()``
+        """
         self.set_format()
 
-    def convert_outputs(self, outputs):
-        if self._format_type is None:
-            if isinstance(outputs, dict) and self._format_columns is not None:
-                return {k: v for k, v in outputs.items() if k in self._format_columns}
+    def _convert_outputs(self, outputs, format_type=None, format_columns=None, output_all_columns=False):
+        if format_type is None:
+            if output_all_columns:
+                return outputs
+            if isinstance(outputs, dict) and format_columns is not None:
+                return {k: v for k, v in outputs.items() if k in format_columns}
             return outputs
 
-        if self._format_type == "numpy":
+        if format_type == "numpy":
             import numpy
 
             command = numpy.array
-        elif self._format_type == "torch":
+        elif format_type == "torch":
             import torch
 
             command = torch.tensor
-        elif self._format_type == "tensorflow":
+        elif format_type == "tensorflow":
             import tensorflow
 
             command = tensorflow.ragged.constant
@@ -236,68 +331,86 @@ class Dataset(object):
 
             command = identity
 
-        try:
-            if isinstance(outputs, (list, tuple)):
-                return command(outputs)
-            return {
-                k: command(v) for k, v in outputs.items() if self._format_columns is None or k in self._format_columns
-            }
-        except TypeError as e:
-            raise TypeError(
-                str(e) + " You should probably filter the output columns with self.set_format(columns=...)."
-            )
+        if isinstance(outputs, (list, tuple)):
+            return command(outputs)
+        else:
+            output_dict = {}
+            for k, v in outputs.items():
+                if format_columns is not None and k not in format_columns and not output_all_columns:
+                    continue
+                if format_columns is None or k in format_columns:
+                    v = command(v)
+                output_dict[k] = v
+        return output_dict
 
     @staticmethod
-    def unnest(py_dict):
+    def _unnest(py_dict):
         return dict((key, array[0]) for key, array in py_dict.items())
 
     @staticmethod
-    def nest(py_dict):
+    def _nest(py_dict):
         return dict((key, [elem]) for key, elem in py_dict.items())
 
-    def __getitem__(self, key: Union[int, slice, str]) -> Union[Dict, List]:
-        """ Can be used to index columns (by string names) or rows (by integer index)
+    def _getitem(
+        self, key: Union[int, slice, str], format_type=None, format_columns=None, output_all_columns=False
+    ) -> Union[Dict, List]:
+        """ Can be used to index columns (by string names) or rows (by integer index or slices)
         """
         if isinstance(key, int):
             if key < 0:
                 key = self._data.num_rows + key
             if key >= self._data.num_rows:
-                raise ValueError(f"Index ({key}) outside of table length ({self._data.num_rows}).")
-            if self._format_type is not None and self._format_type == "pandas":
+                raise IndexError(f"Index ({key}) outside of table length ({self._data.num_rows}).")
+            if format_type is not None and format_type == "pandas":
                 outputs = self._data.slice(key, 1).to_pandas()
             else:
-                outputs = self.unnest(self._data.slice(key, 1).to_pydict())
+                outputs = self._unnest(self._data.slice(key, 1).to_pydict())
         elif isinstance(key, slice):
-            key = key.indices(self._data.num_rows)
-            if key[2] != 1 or key[1] < key[0]:
+            key_indices = key.indices(self._data.num_rows)
+            if key_indices[2] != 1 or key_indices[1] < key_indices[0]:
                 raise ValueError("Slicing can only take contiguous and ordered slices.")
-            if self._format_type is not None and self._format_type == "pandas":
-                outputs = self._data.slice(key[0], key[1] - key[0]).to_pandas()
+            if format_type is not None and format_type == "pandas":
+                outputs = self._data.slice(key_indices[0], key_indices[1] - key_indices[0]).to_pandas()
             else:
-                outputs = self._data.slice(key[0], key[1] - key[0]).to_pydict()
+                outputs = self._data.slice(key_indices[0], key_indices[1] - key_indices[0]).to_pydict()
         elif isinstance(key, str):
             if key not in self._data.column_names:
                 raise ValueError(f"Column ({key}) not in table columns ({self._data.column_names}).")
-            if self._format_type is not None:
-                if self._format_type == "pandas":
-                    outputs = self._data[key].to_pandas()
-                elif self._format_type == "numpy":
-                    outputs = self._data[key].to_numpy()
+            if format_type is not None:
+                if format_columns is None or key in format_columns:
+                    if format_type == "pandas":
+                        outputs = self._data[key].to_pandas()
+                    elif format_type == "numpy":
+                        outputs = np.concatenate([arr.to_numpy() for arr in self._data[key].chunks])
+                    else:
+                        outputs = self._convert_outputs(self._data[key].to_pylist(), format_type=format_type)
                 else:
-                    outputs = self.convert_outputs(self._data[key].to_pylist())
+                    outputs = self._data[key].to_pylist()
             else:
                 outputs = self._data[key].to_pylist()
         else:
             raise ValueError("Can only get row(s) (int or slice) or columns (string).")
 
         if (
-            (self._format_type is not None or self._format_columns is not None)
+            (format_type is not None or format_columns is not None)
             and not isinstance(key, str)
-            and self._format_type != "pandas"
+            and format_type != "pandas"
         ):
-            outputs = self.convert_outputs(outputs)
+            outputs = self._convert_outputs(
+                outputs, format_type=format_type, format_columns=format_columns, output_all_columns=output_all_columns
+            )
         return outputs
 
+    def __getitem__(self, key: Union[int, slice, str]) -> Union[Dict, List]:
+        """ Can be used to index columns (by string names) or rows (by integer index)
+        """
+        return self._getitem(
+            key,
+            format_type=self._format_type,
+            format_columns=self._format_columns,
+            output_all_columns=self._output_all_columns,
+        )
+
     def cleanup_cache_files(self):
         """ Clean up all cache files in the dataset cache directory, excepted the currently used cache file if there is one.
             Be carefull when running this command that no other process is currently using other cache files.
@@ -325,12 +438,11 @@ class Dataset(object):
         return len(files_to_remove)
 
     def _get_cache_file_path(self, function, cache_kwargs):
-        """ Find a unique name from the filenames, kwargs and the function
-        """
+        """ Find a unique name from the filenames, kwargs and the function """
         if not self._data_files or "filename" not in self._data_files[0]:
             return None
         previous_files_string = "-".join(
-            "-".join(str(k) + "-" + str(v) for k, v in cache_kwargs.items()) for f in self._data_files
+            "-".join(str(k) + "-" + str(v) for k, v in f.items()) for f in self._data_files
         )
         cache_kwargs_string = "-".join(str(k) + "-" + str(v) for k, v in cache_kwargs.items())
         function_bytes = dumps(function)
@@ -365,7 +477,7 @@ class Dataset(object):
                     - `function(example: Dict, indices: int) -> Union[Dict, Any]` if `batched=False` and `with_indices=True`
                     - `function(batch: Dict[List]) -> Union[Dict, Any]` if `batched=True` and `with_indices=False`
                     - `function(batch: Dict[List], indices: List[int]) -> Union[Dict, Any]` if `batched=True` and `with_indices=True`
-                `with_indices` (`bool`, default: `False`): Provide example indices to `function`
+                `with_indices` (`bool`, default: `False`): Provide example indices to `function`. Note that in this case the signature of `function` should be `def function(example, idx): ...`.
                 `batched` (`bool`, default: `False`): Provide batch of examples to `function`
                 `batch_size` (`Optional[int]`, default: `1000`): Number of examples per batch provided to `function` if `batched=True`
                     `batch_size <= 0` or `batch_size == None`: Provide the full dataset as a single batch to `function`
@@ -437,6 +549,12 @@ class Dataset(object):
             if remove_columns is not None:
                 for column in remove_columns:
                     inputs.pop(column)
+            if self._format_type is not None:
+                inputs = self._getitem(
+                    key=(indices if isinstance(indices, int) else slice(indices[0], indices[-1])),
+                    format_type=None,
+                    format_columns=None,
+                )
             inputs.update(processed_inputs)
             return inputs
 
@@ -446,7 +564,7 @@ class Dataset(object):
         test_output = apply_function_on_filtered_inputs(test_inputs, test_indices)
         if arrow_schema is None and update_data:
             if not batched:
-                test_output = self.nest(test_output)
+                test_output = self._nest(test_output)
             test_output = convert_tuples_in_lists(test_output)
             arrow_schema = pa.Table.from_pydict(test_output).schema
             if disable_nullable:
@@ -471,7 +589,7 @@ class Dataset(object):
                 cache_file_name = self._get_cache_file_path(function, cache_kwargs)
             if os.path.exists(cache_file_name) and load_from_cache_file:
                 logger.info("Loading cached processed dataset at %s", cache_file_name)
-                return Dataset.from_file(cache_file_name)
+                return Dataset.from_file(cache_file_name, info=self.info, split=self.split)
 
         # Prepare output buffer and batched writer in memory or on file if we update the table
         if update_data:
@@ -502,8 +620,71 @@ class Dataset(object):
 
             # Create new Dataset from buffer or file
             if buf_writer is None:
-                return Dataset.from_file(cache_file_name)
+                return Dataset.from_file(cache_file_name, info=self.info, split=self.split)
             else:
-                return Dataset.from_buffer(buf_writer.getvalue())
+                return Dataset.from_buffer(buf_writer.getvalue(), info=self.info, split=self.split)
         else:
             return self
+
+    def filter(self, function, with_indices=False, **kwargs):
+        """ Apply a filter function to all the elements in the table in batches
+            and update the table so that the dataset only includes examples according to the filter function.
+
+            Args:
+                `function` (`callable`): with one of the following signature:
+                    - `function(example: Dict) -> bool` if `with_indices=False`
+                    - `function(example: Dict, indices: int) -> bool` if `with_indices=True`
+                `with_indices` (`bool`, default: `False`): Provide example indices to `function`. Note that in this case the signature of `function` should be `def function(example, idx): ...`.
+                `batch_size` (`Optional[int]`, default: `1000`): Number of examples per batch provided to `function` if `batched=True`
+                    `batch_size <= 0` or `batch_size == None`: Provide the full dataset as a single batch to `function`
+                `remove_columns` (`Optional[List[str]]`, default: `None`): Remove a selection of columns while doing the mapping.
+                    Columns will be removed before updating the examples with the output of `function`, i.e. if `function` is adding
+                    columns with names in `remove_columns`, these columns will be kept.
+                `keep_in_memory` (`bool`, default: `False`): Keep the dataset in memory instead of writing it to a cache file.
+                `load_from_cache_file` (`bool`, default: `True`): If a cache file storing the current computation from `function`
+                    can be identified, use it instead of recomputing.
+                `cache_file_name` (`Optional[str]`, default: `None`): Provide the name of a cache file to use to store the
+                    results of the computation instead of the automatically generated cache file name.
+                `writer_batch_size` (`int`, default: `1000`): Number of rows per write operation for the cache file writer.
+                    Higher value gives smaller cache files, lower value consume less temporary memory while running `.map()`.
+                `disable_nullable` (`bool`, default: `True`): Allow null values in the table.
+        """
+
+        # transforme the filter function into the map function
+        def map_function(batch, *args):
+            result = defaultdict(list)
+            num_examples = len(batch[next(iter(batch.keys()))])
+
+            # create single examples
+            for i in range(num_examples):
+                example = map_nested(lambda x: x[i], batch, dict_only=True)
+
+                # check if example should be fildered or not
+                if with_indices:
+                    keep_example = function(example, args[0][i])
+                else:
+                    keep_example = function(example)
+
+                assert isinstance(
+                    keep_example, bool
+                ), f"The filter function returns a variable of type {type(keep_example)}, but should return a variable of type `bool`."
+                # if example shall be kept add to result
+                if keep_example:
+                    for key in batch.keys():
+                        result[key].append(example[key])
+
+            # if no example shall be kept, init with empty list
+            if bool(result) is False:
+                for key in batch.keys():
+                    result[key] = []
+
+            return result
+
+        # to avoid errors with the arrow_schema we define it here
+        test_inputs = self[:2]
+        if "remove_columns" in kwargs:
+            test_inputs = {key: test_inputs[key] for key in (test_inputs.keys() - kwargs["remove_columns"])}
+        arrow_schema = pa.Table.from_pydict(test_inputs).schema
+
+        # return map function
+        return self.map(map_function, batched=True, with_indices=with_indices, arrow_schema=arrow_schema, **kwargs)
