@@ -17,6 +17,7 @@
 """ Arrow ArrowReader."""
 
 import copy
+import logging
 import math
 import os
 import re
@@ -28,10 +29,13 @@ import pyarrow.parquet
 
 from .arrow_dataset import Dataset
 from .naming import filename_for_dataset_split
-from .utils import py_utils
+from .utils import cached_path, py_utils
 
 
+logger = logging.getLogger(__name__)
+
 _BUFFER_SIZE = 8 << 20  # 8 MiB per file.
+HF_GCP_BASE_URL = "https://storage.googleapis.com/huggingface-nlp/cache/datasets"
 
 _SUB_SPEC_RE = re.compile(
     r"""
@@ -52,6 +56,12 @@ $
 _ADDITION_SEP_RE = re.compile(r"\s*\+\s*")
 
 
+class DatasetNotOnHfGcs(ConnectionError):
+    """When you can't get the dataset from the Hf google cloud storage"""
+
+    pass
+
+
 @dataclass(frozen=True)
 class FileInstructions:
     """The file instructions associated with a split ReadInstruction.
@@ -128,13 +138,14 @@ class BaseReader:
         """Returns a Dataset instance from given (filename, skip, take)."""
         raise NotImplementedError
 
-    def _read_files(self, files, info) -> Dataset:
+    def _read_files(self, files, info, original_instructions) -> Dataset:
         """Returns Dataset for given file instructions.
 
         Args:
             files: List[dict(filename, skip, take)], the files information.
                 The filenames contain the absolute path, not relative.
                 skip/take indicates which example read in the file: `ds.slice(skip, take)`
+            original_instructions: store the original instructions used to build the dataset split in the dataset
         """
         pa_batches = []
         for f_dict in files:
@@ -142,9 +153,17 @@ class BaseReader:
             pa_batches.extend(pa_table.to_batches())
         if pa_batches:
             pa_table = pa.Table.from_batches(pa_batches)
-        ds = Dataset(arrow_table=pa_table, data_files=files, info=info)
+        ds = Dataset(arrow_table=pa_table, data_files=files, info=info, split=original_instructions)
         return ds
 
+    def get_file_instructions(self, name, instruction, split_infos):
+        """Return list of dict {'filename': str, 'skip': int, 'take': int}"""
+        file_instructions = make_file_instructions(
+            name, split_infos, instruction, filetype_suffix=self._filetype_suffix
+        )
+        files = file_instructions.file_instructions
+        return files
+
     def read(
         self, name, instructions, split_infos,
     ):
@@ -164,19 +183,16 @@ class BaseReader:
         """
 
         def _read_instruction_to_ds(instruction):
-            file_instructions = make_file_instructions(
-                name, split_infos, instruction, filetype_suffix=self._filetype_suffix
-            )
-            files = file_instructions.file_instructions
+            files = self.get_file_instructions(name, instruction, split_infos)
             if not files:
                 msg = 'Instruction "%s" corresponds to no data!' % instruction
                 raise AssertionError(msg)
-            return self.read_files(files=tuple(files),)
+            return self.read_files(files=tuple(files), original_instructions=instruction)
 
         return py_utils.map_nested(_read_instruction_to_ds, instructions)
 
     def read_files(
-        self, files,
+        self, files, original_instructions,
     ):
         """Returns single Dataset instance for the set of file instructions.
 
@@ -184,6 +200,7 @@ class BaseReader:
             files: List[dict(filename, skip, take)], the files information.
                 The filenames contains the relative path, not absolute.
                 skip/take indicates which example read in the file: `ds.skip().take()`
+            original_instructions: store the original instructions used to build the dataset split in the dataset
 
         Returns:
              a Dataset instance.
@@ -192,9 +209,35 @@ class BaseReader:
         files = copy.deepcopy(files)
         for f in files:
             f.update(filename=os.path.join(self._path, f["filename"]))
-        dataset = self._read_files(files=files, info=self._info,)
+        dataset = self._read_files(files=files, info=self._info, original_instructions=original_instructions)
         return dataset
 
+    def download_from_hf_gcs(self, cache_dir, relative_data_dir):
+        """
+        Download the dataset files from the Hf GCS
+
+        Args:
+            cache_dir: `str`, the local cache directory where to save the dataset
+            relative_data_dir: `str`, the relative directory of the remote files from
+                the `datasets` directory on GCS.
+
+        """
+        remote_cache_dir = os.path.join(HF_GCP_BASE_URL, relative_data_dir)
+        try:
+            remote_dataset_info = os.path.join(remote_cache_dir, "dataset_info.json")
+            downloaded_dataset_info = cached_path(remote_dataset_info)
+            os.rename(downloaded_dataset_info, os.path.join(cache_dir, "dataset_info.json"))
+        except ConnectionError:
+            raise DatasetNotOnHfGcs()
+        for split in self._info.splits:
+            file_instructions = self.get_file_instructions(
+                name=self._info.builder_name, instruction=split, split_infos=self._info.splits.values(),
+            )
+            for file_instruction in file_instructions:
+                remote_prepared_filename = os.path.join(remote_cache_dir, file_instruction["filename"])
+                downloaded_prepared_filename = cached_path(remote_prepared_filename)
+                os.rename(downloaded_prepared_filename, os.path.join(cache_dir, file_instruction["filename"]))
+
 
 class ArrowReader(BaseReader):
     """
@@ -355,26 +398,26 @@ class ReadInstruction(object):
 
     ```
     # The following lines are equivalent:
-    ds = nlp.load('mnist', split='test[:33%]')
-    ds = nlp.load('mnist', split=ReadInstruction.from_spec('test[:33%]'))
-    ds = nlp.load('mnist', split=ReadInstruction('test', to=33, unit='%'))
-    ds = nlp.load('mnist', split=ReadInstruction(
+    ds = nlp.load_dataset('mnist', split='test[:33%]')
+    ds = nlp.load_dataset('mnist', split=ReadInstruction.from_spec('test[:33%]'))
+    ds = nlp.load_dataset('mnist', split=ReadInstruction('test', to=33, unit='%'))
+    ds = nlp.load_dataset('mnist', split=ReadInstruction(
             'test', from_=0, to=33, unit='%'))
 
     # The following lines are equivalent:
-    ds = nlp.load('mnist', split='test[:33%]+train[1:-1]')
-    ds = nlp.load('mnist', split=ReadInstruction.from_spec(
+    ds = nlp.load_dataset('mnist', split='test[:33%]+train[1:-1]')
+    ds = nlp.load_dataset('mnist', split=ReadInstruction.from_spec(
             'test[:33%]+train[1:-1]'))
-    ds = nlp.load('mnist', split=(
+    ds = nlp.load_dataset('mnist', split=(
             ReadInstruction.('test', to=33, unit='%') +
             ReadInstruction.('train', from_=1, to=-1, unit='abs')))
 
     # 10-fold validation:
-    tests = nlp.load(
+    tests = nlp.load_dataset(
             'mnist',
             [ReadInstruction('train', from_=k, to=k+10, unit='%')
              for k in range(0, 100, 10)])
-    trains = nlp.load(
+    trains = nlp.load_dataset(
             'mnist',
             [RI('train', to=k, unit='%') + RI('train', from_=k+10, unit='%')
              for k in range(0, 100, 10)])
