@@ -25,16 +25,18 @@ import shutil
 from dataclasses import dataclass
 from typing import Dict, List, Optional, Union
 
-from . import splits as splits_lib
+import pyarrow as pa
+
 from . import utils
-from .arrow_reader import ArrowReader, ParquetReader
+from .arrow_reader import ArrowReader, DatasetNotOnHfGcs
 from .arrow_writer import ArrowWriter, BeamWriter
 from .features import Features, Value
-from .info import DatasetInfo
+from .info import DATASET_INFO_FILENAME, DATASET_INFOS_DICT_FILE_NAME, LICENSE_FILENAME, DatasetInfo, DatasetInfosDict
 from .naming import camelcase_to_snakecase, filename_prefix_for_split
-from .utils.checksums_utils import URLS_CHECKSUMS_FOLDER_NAME
+from .splits import Split, SplitDict
 from .utils.download_manager import DownloadManager, GenerateMode
-from .utils.file_utils import HF_DATASETS_CACHE, DownloadConfig
+from .utils.file_utils import HF_DATASETS_CACHE, DownloadConfig, is_remote_url
+from .utils.info_utils import verify_checksums, verify_splits
 
 
 logger = logging.getLogger(__name__)
@@ -44,6 +46,10 @@ REUSE_CACHE_IF_EXISTS = GenerateMode.REUSE_CACHE_IF_EXISTS
 REUSE_DATASET_IF_EXISTS = GenerateMode.REUSE_DATASET_IF_EXISTS
 
 
+class InvalidConfigName(ValueError):
+    pass
+
+
 @dataclass
 class BuilderConfig:
     """Base class for `DatasetBuilder` data configuration.
@@ -52,11 +58,24 @@ class BuilderConfig:
     `BuilderConfig` and add their own properties.
     """
 
-    name: str = "no-name"
-    version: Optional[Union[str, utils.Version]] = "no-version"
-    description: Optional[str] = None
+    name: str = "default"
+    version: Optional[Union[str, utils.Version]] = "0.0.0"
     data_dir: str = None
     data_files: Union[Dict, List] = None
+    description: str = None
+
+    def __post_init__(self):
+        # The config name is used to name the cache directory.
+        invalid_windows_characters = r"<>:/\|?*"
+        for invalid_char in invalid_windows_characters:
+            if invalid_char in self.name:
+                raise InvalidConfigName(
+                    (
+                        "Bad characters from black list '{}' found in '{}'. "
+                        "They could create issues when creating a directory "
+                        "for this config on Windows filesystem."
+                    ).format(invalid_windows_characters, self.name)
+                )
 
 
 class DatasetBuilder:
@@ -74,26 +93,14 @@ class DatasetBuilder:
     dataset by defining a `nlp.BuilderConfig` subclass and accepting a
     config object (or name) on construction. Configurable datasets expose a
     pre-defined set of configurations in `nlp.DatasetBuilder.builder_configs`.
-
-    Typical `DatasetBuilder` usage:
-
-    ```python
-    mnist_builder = nlp.get_dataset_builder("mnist")
-    mnist_info = mnist_builder.info
-    mnist_builder.download_and_prepare()
-    datasets = mnist_builder.as_dataset()
-
-    train_dataset, test_dataset = datasets["train"], datasets["test"]
-    assert isinstance(train_dataset, Dataset`)
-
-    # And then the rest of your input pipeline
-    train_dataset = train_dataset.repeat().shuffle(1024).batch(128)
-    train_dataset = train_dataset.prefetch(2)
-    features = tf.compat.v1.data.make_one_shot_iterator(train_dataset).get_next()
-    image, label = features['image'], features['label']
-    ```
     """
 
+    # Default version.
+    VERSION = utils.Version("0.0.0")
+
+    # Class for the builder config.
+    BUILDER_CONFIG_CLASS = BuilderConfig
+
     # Named configurations that modify the data generated by download_and_prepare.
     BUILDER_CONFIGS = []
 
@@ -107,17 +114,15 @@ class DatasetBuilder:
     MANUAL_DOWNLOAD_INSTRUCTIONS = None
 
     def __init__(
-        self, cache_dir=None, config=None, **config_kwargs,
+        self, cache_dir=None, name=None, **config_kwargs,
     ):
         """Constructs a DatasetBuilder.
 
         Callers must pass arguments as keyword arguments.
 
         Args:
-            cache_dir: `str`, directory to read/write data. Defaults to
-                "~/nlp".
-            config: `nlp.BuilderConfig` or `str` name, optional configuration
-                for the dataset that affects the data generated on disk. Different
+            cache_dir: `str`, directory to read/write data. Defaults to "~/nlp".
+            name: `str` name, optional configuration for the dataset that affects the data generated on disk. Different
                 `builder_config`s will have their own subdirectories and versions.
                 If not provided, uses the first configuration in self.BUILDER_CONFIGS
             config_kwargs: will override the defaults kwargs in config
@@ -128,44 +133,76 @@ class DatasetBuilder:
 
         # Prepare config: DatasetConfig contains name, version and description but can be extended by each dataset
         config_kwargs = dict((key, value) for key, value in config_kwargs.items() if value is not None)
-        self.config = self._create_builder_config(config, **config_kwargs,)
+        self.config = self._create_builder_config(name, **config_kwargs,)
 
         # prepare info: DatasetInfo are a standardized dataclass across all datasets
-        info = self._info()
+        # Prefill datasetinfo
+        info = self.get_exported_dataset_info()
+        info.update(self._info())
         info.builder_name = self.name
         info.config_name = self.config.name
         info.version = self.config.version
-        info.splits.dataset_name = self.name
         self.info = info
-        urls_checksums_dir = os.path.join(self.get_imported_module_dir(), URLS_CHECKSUMS_FOLDER_NAME)
-        self.info.prefill_dataset_size_attributes_from_urls_checksums_dir(urls_checksums_dir)
 
         # prepare data dirs
         self._cache_dir_root = os.path.expanduser(cache_dir or HF_DATASETS_CACHE)
         self._cache_dir = self._build_cache_dir()
-        if os.path.exists(self._cache_dir):
-            logger.info("Overwrite dataset info from restored data version.")
-            self.info = DatasetInfo.from_directory(self._cache_dir)
+        if os.path.exists(self._cache_dir):  # check if data exist
+            if len(os.listdir(self._cache_dir)) > 0:
+                logger.info("Overwrite dataset info from restored data version.")
+                self.info = DatasetInfo.from_directory(self._cache_dir)
+            else:  # dir exists but no data, remove the empty dir as data aren't available anymore
+                logger.warning(
+                    "Old caching folder {} for dataset {} exists but not data were found. Removing it. ".format(
+                        self._cache_dir, self.name
+                    )
+                )
+                os.rmdir(self._cache_dir)
 
-    def _create_builder_config(self, builder_config=None, **config_kwargs):
+    @property
+    def does_require_manual_download(self):
+        return hasattr(self, "MANUAL_DOWNLOAD_INSTRUCTIONS") and (self.MANUAL_DOWNLOAD_INSTRUCTIONS is not None)
+
+    @classmethod
+    def get_all_exported_dataset_infos(cls) -> dict:
+        """Empty dict if doesn't exist"""
+        dset_infos_file_path = os.path.join(cls.get_imported_module_dir(), DATASET_INFOS_DICT_FILE_NAME)
+        if os.path.exists(dset_infos_file_path):
+            return DatasetInfosDict.from_directory(cls.get_imported_module_dir())
+        return {}
+
+    def get_exported_dataset_info(self) -> DatasetInfo:
+        """Empty DatasetInfo if doesn't exist"""
+        return self.get_all_exported_dataset_infos().get(self.config.name, DatasetInfo())
+
+    def _create_builder_config(self, name=None, **config_kwargs):
         """ Create and validate BuilderConfig object.
-            Uses the first configuration in self.BUILDER_CONFIGS if builder_config is None
+            Uses the first configuration in self.BUILDER_CONFIGS if name is None
             config_kwargs override the defaults kwargs in config
         """
-        if builder_config is None and self.BUILDER_CONFIGS:
+        builder_config = None
+        if name is None and self.BUILDER_CONFIGS and not config_kwargs:
+            if len(self.BUILDER_CONFIGS) > 1:
+                example_of_usage = "load_dataset('{}', '{}')".format(self.name, self.BUILDER_CONFIGS[0].name)
+                raise ValueError(
+                    "Config name is missing."
+                    "\nPlease pick one among the available configs: %s" % list(self.builder_configs.keys())
+                    + "\nExample of usage:\n\t`{}`".format(example_of_usage)
+                )
             builder_config = self.BUILDER_CONFIGS[0]
             logger.info("No config specified, defaulting to first: %s/%s", self.name, builder_config.name)
-        if isinstance(builder_config, str):
-            name = builder_config
+        if isinstance(name, str):
             builder_config = self.builder_configs.get(name)
-            if builder_config is None:
+            if builder_config is None and self.BUILDER_CONFIGS:
                 raise ValueError(
                     "BuilderConfig %s not found. Available: %s" % (name, list(self.builder_configs.keys()))
                 )
         if not builder_config:
-            builder_config = BuilderConfig()
-            if hasattr(self, "VERSION") and self.VERSION:
-                builder_config.version = self.VERSION
+            if name is not None:
+                config_kwargs["name"] = name
+            if "version" not in config_kwargs and hasattr(self, "VERSION") and self.VERSION:
+                config_kwargs["version"] = self.VERSION
+            builder_config = self.BUILDER_CONFIG_CLASS(**config_kwargs)
 
         for key, value in config_kwargs.items():
             if value is not None:
@@ -275,22 +312,23 @@ class DatasetBuilder:
 
     def download_and_prepare(
         self,
-        download_config=None,
-        download_mode=None,
-        ignore_checksums=False,
-        save_checksums=False,
-        dl_manager=None,
+        download_config: Optional[DownloadConfig] = None,
+        download_mode: Optional[GenerateMode] = None,
+        ignore_verifications: bool = False,
+        save_infos: bool = False,
+        try_from_hf_gcs: bool = True,
+        dl_manager: Optional[DownloadManager] = None,
         **download_and_prepare_kwargs,
     ):
         """Downloads and prepares dataset for reading.
 
         Args:
-            cache_dir: `str`, directory where downloaded files are stored.
-            dl_manager: `nlp.DownloadConfig`, further configuration for
-                downloading and preparing dataset.
-
-        Raises:
-            IOError: if there is not enough disk space available.
+            download_config (Optional ``nlp.DownloadConfig``: specific download configuration parameters.
+            download_mode (Optional `nlp.GenerateMode`): select the download/generate mode - Default to REUSE_DATASET_IF_EXISTS
+            ignore_verifications (bool): Ignore the verifications of the downloaded/processed dataset information (checksums/size/splits/...)
+            save_infos (bool): Save the dataset information (checksums/size/splits/...)
+            try_from_hf_gcs (bool): If True, it will try to download the already prepared dataset from the Hf google cloud storage
+            dl_manager (Optional ``nlp.DownloadManager``): specific Download Manger to use
         """
         download_mode = GenerateMode(download_mode or GenerateMode.REUSE_DATASET_IF_EXISTS)
 
@@ -310,16 +348,54 @@ class DatasetBuilder:
             )
 
         logger.info("Generating dataset %s (%s)", self.name, self._cache_dir)
-        if not utils.has_sufficient_disk_space(
-            self.info.dataset_size + self.info.download_size, directory=self._cache_dir_root
-        ):
-            raise IOError(
-                "Not enough disk space. Needed: {} (download: {}, generated: {})".format(
-                    utils.size_str(self.info.dataset_size + self.info.download_size),
-                    utils.size_str(self.info.download_size),
-                    utils.size_str(self.info.dataset_size),
+        if not is_remote_url(self._cache_dir):  # if cache dir is local, check for available space
+            os.makedirs(self._cache_dir_root, exist_ok=True)
+            if not utils.has_sufficient_disk_space(self.info.size_in_bytes or 0, directory=self._cache_dir_root):
+                raise IOError(
+                    "Not enough disk space. Needed: {} (download: {}, generated: {})".format(
+                        utils.size_str(self.info.size_in_bytes or 0),
+                        utils.size_str(self.info.download_size or 0),
+                        utils.size_str(self.info.dataset_size or 0),
+                    )
                 )
-            )
+
+        @contextlib.contextmanager
+        def incomplete_dir(dirname):
+            """Create temporary dir for dirname and rename on exit."""
+            if is_remote_url(dirname):
+                yield dirname
+            else:
+                tmp_dir = dirname + ".incomplete"
+                os.makedirs(tmp_dir)
+                try:
+                    yield tmp_dir
+                    if os.path.isdir(dirname):
+                        shutil.rmtree(dirname)
+                    os.rename(tmp_dir, dirname)
+                finally:
+                    if os.path.exists(tmp_dir):
+                        shutil.rmtree(tmp_dir)
+
+        # Try to download the already prepared dataset files
+        if try_from_hf_gcs:
+            try:
+                # Create a tmp dir and rename to self._cache_dir on successful exit.
+                with incomplete_dir(self._cache_dir) as tmp_data_dir:
+                    # Temporarily assign _cache_dir to tmp_data_dir to avoid having to forward
+                    # it to every sub function.
+                    with utils.temporary_assignment(self, "_cache_dir", tmp_data_dir):
+                        reader = ArrowReader(self._cache_dir, self.info)
+                        reader.download_from_hf_gcs(self._cache_dir, self._relative_data_dir(with_version=True))
+                        downloaded_info = DatasetInfo.from_directory(self._cache_dir)
+                        self.info.update(downloaded_info)
+                logger.info("Dataset downloaded from Hf google storage.")
+                print(
+                    f"Dataset {self.name} downloaded and prepared to {self._cache_dir}. "
+                    f"Subsequent calls will reuse this data."
+                )
+                return
+            except DatasetNotOnHfGcs:
+                logger.info("Dataset not on Hf google storage. Downloading and preparing it from source")
 
         # Print is intentional: we want this to always go to stdout so user has
         # information needed to cancel download/preparation if needed.
@@ -327,53 +403,52 @@ class DatasetBuilder:
         print(
             f"Downloading and preparing dataset {self.info.builder_name}/{self.info.config_name} "
             f"(download: {utils.size_str(self.info.download_size)}, generated: {utils.size_str(self.info.dataset_size)}, "
-            f"total: {utils.size_str(self.info.download_size + self.info.dataset_size)}) to {self._cache_dir}..."
+            f"total: {utils.size_str(self.info.size_in_bytes)}) to {self._cache_dir}..."
         )
 
         if dl_manager is None:
-            download_config = DownloadConfig()
-            download_config.cache_dir = os.path.join(self._cache_dir_root, "downloads")
-            download_config.force_download = download_mode == FORCE_REDOWNLOAD
+            if download_config is None:
+                download_config = DownloadConfig()
+                download_config.cache_dir = os.path.join(self._cache_dir_root, "downloads")
+                download_config.force_download = download_mode == FORCE_REDOWNLOAD
 
             dl_manager = DownloadManager(
-                dataset_name=self.name,
-                ignore_checksums=ignore_checksums,
-                save_checksums=save_checksums,
-                download_config=download_config,
+                dataset_name=self.name, download_config=download_config, data_dir=self.config.data_dir
             )
 
-        @contextlib.contextmanager
-        def incomplete_dir(dirname):
-            """Create temporary dir for dirname and rename on exit."""
-            tmp_dir = dirname + ".incomplete"
-            os.makedirs(tmp_dir)
-            try:
-                yield tmp_dir
-                if os.path.isdir(dirname):
-                    shutil.rmtree(dirname)
-                os.rename(tmp_dir, dirname)
-            finally:
-                if os.path.exists(tmp_dir):
-                    shutil.rmtree(tmp_dir)
+        if self.does_require_manual_download:
+            assert (
+                dl_manager.manual_dir is not None
+            ), "The dataset {} with config {} requires manual data. \n Please follow the manual download instructions: {}. \n Manual data can be loaded with `nlp.load({}, data_dir='<path/to/manual/data>')".format(
+                self.name, self.config.name, self.MANUAL_DOWNLOAD_INSTRUCTIONS, self.name
+            )
 
         # Create a tmp dir and rename to self._cache_dir on successful exit.
         with incomplete_dir(self._cache_dir) as tmp_data_dir:
             # Temporarily assign _cache_dir to tmp_data_dir to avoid having to forward
             # it to every sub function.
             with utils.temporary_assignment(self, "_cache_dir", tmp_data_dir):
-                self._download_and_prepare(dl_manager=dl_manager, **download_and_prepare_kwargs)
-                urls_checksums_dir = os.path.join(self.get_imported_module_dir(), URLS_CHECKSUMS_FOLDER_NAME)
-                self.info.check_or_save_cached_sizes(
-                    urls_checksums_dir, ignore_cached_sizes=ignore_checksums, save_cached_sizes=save_checksums
+                verify_infos = not save_infos and not ignore_verifications
+                self._download_and_prepare(
+                    dl_manager=dl_manager, verify_infos=verify_infos, **download_and_prepare_kwargs
                 )
-                self.info.write_to_directory(self._cache_dir)
+                # Sync info
+                self.info.dataset_size = sum(split.num_bytes for split in self.info.splits.values())
+                self.info.download_checksums = dl_manager.get_recorded_sizes_checksums()
+                self.info.size_in_bytes = self.info.dataset_size + self.info.download_size
+                # Save info
+                self._save_info()
+
+        # Save to datasetinfos
+        if save_infos:
+            DatasetInfosDict(**{self.config.name: self.info}).write_to_directory(self.get_imported_module_dir())
 
         print(
             f"Dataset {self.name} downloaded and prepared to {self._cache_dir}. "
             f"Subsequent calls will reuse this data."
         )
 
-    def _download_and_prepare(self, dl_manager, **prepare_split_kwargs):
+    def _download_and_prepare(self, dl_manager, verify_infos, **prepare_split_kwargs):
         """Downloads and prepares dataset for reading.
 
         This is the internal implementation to overwrite called when user calls
@@ -383,16 +458,16 @@ class DatasetBuilder:
         Args:
             dl_manager: (DownloadManager) `DownloadManager` used to download and cache
                 data.
-            download_config: `DownloadConfig`, Additional options.
+            verify_infos: bool, if True, do not perform checksums and size tests.
+            prepare_split_kwargs: Additional options.
         """
-        os.makedirs(self._cache_dir, exist_ok=True)
-        urls_checksums_dir = os.path.join(self.get_imported_module_dir(), URLS_CHECKSUMS_FOLDER_NAME)
-
         # Generating data for all splits
-        split_dict = splits_lib.SplitDict(dataset_name=self.name)
+        split_dict = SplitDict(dataset_name=self.name)
         split_generators_kwargs = self._make_split_generators_kwargs(prepare_split_kwargs)
         split_generators = self._split_generators(dl_manager, **split_generators_kwargs)
-        dl_manager.check_or_save_checksums(urls_checksums_dir)  # verify checksums
+        # Checksums verification
+        if verify_infos:
+            verify_checksums(self.info.download_checksums, dl_manager.get_recorded_sizes_checksums())
         for split_generator in split_generators:
             if str(split_generator.split_info.name).lower() == "all":
                 raise ValueError(
@@ -410,18 +485,23 @@ class DatasetBuilder:
             except OSError:
                 raise OSError("Cannot find data file. " + (self.MANUAL_DOWNLOAD_INSTRUCTIONS or ""))
 
+        if verify_infos:
+            verify_splits(self.info.splits, split_dict)
         # Update the info object with the splits.
-        self.info.splits = split_dict.copy()
+        self.info.splits = split_dict
         self.info.download_size = dl_manager.downloaded_size
-        self.info.dataset_size = sum(split.num_bytes for split in self.info.splits.values())
-        self.info.download_checksums = dl_manager.get_recorded_sizes_checksums()
+
+    def _save_info(self):
+        self.info.write_to_directory(self._cache_dir)
 
     def _make_split_generators_kwargs(self, prepare_split_kwargs):
         """Get kwargs for `self._split_generators()` from `prepare_split_kwargs`."""
         del prepare_split_kwargs
         return {}
 
-    def as_dataset(self, split=None):
+    def as_dataset(self, split: Optional[Split] = None):
+        """ Return a Dataset for the specified split.
+        """
         logger.info("Constructing Dataset for split %s, from %s", split, self._cache_dir)
         if not os.path.exists(self._cache_dir):
             raise AssertionError(
@@ -444,13 +524,13 @@ class DatasetBuilder:
     def _build_single_dataset(self, split):
         """as_dataset for a single split."""
         if isinstance(split, str):
-            split = splits_lib.Split(split)
+            split = Split(split)
 
         # Build base dataset
         ds = self._as_dataset(split=split,)
         return ds
 
-    def _as_dataset(self, split=splits_lib.Split.TRAIN):
+    def _as_dataset(self, split: Split = Split.TRAIN):
         """Constructs a `Dataset`.
 
         This is the internal implementation to overwrite called when user calls
@@ -536,6 +616,13 @@ class GeneratorBasedBuilder(DatasetBuilder):
     (`_split_generators`). See the method docstrings for details.
     """
 
+    # GeneratorBasedBulder should have dummy data for tests by default
+    test_dummy_data = True
+
+    def __init__(self, *args, **kwargs):
+        super(GeneratorBasedBuilder, self).__init__(*args, **kwargs)
+        self._writer_batch_size = kwargs.get("writer_batch_size")
+
     @abc.abstractmethod
     def _generate_examples(self, **kwargs):
         """Default function generating examples for each `SplitGenerator`.
@@ -572,7 +659,7 @@ class GeneratorBasedBuilder(DatasetBuilder):
         fname = "{}-{}.arrow".format(self.name, split_generator.name)
         fpath = os.path.join(self._cache_dir, fname)
         examples_type = self.info.features.type
-        writer = ArrowWriter(data_type=examples_type, path=fpath)
+        writer = ArrowWriter(data_type=examples_type, path=fpath, writer_batch_size=self._writer_batch_size)
 
         generator = self._generate_examples(**split_generator.gen_kwargs)
         for key, record in utils.tqdm(generator, unit=" examples", total=split_info.num_examples, leave=False):
@@ -590,6 +677,9 @@ class ArrowBasedBuilder(DatasetBuilder):
 
     """
 
+    # ArrowBasedBuilder should have dummy data for tests by default
+    test_dummy_data = True
+
     @abc.abstractmethod
     def _generate_examples(self, **kwargs):
         """Default function generating examples for each `SplitGenerator`.
@@ -633,20 +723,37 @@ class ArrowBasedBuilder(DatasetBuilder):
 
         split_generator.split_info.num_examples = num_examples
         split_generator.split_info.num_bytes = num_bytes
-        self.info.features = Features(
-            {
-                field.name: Value(str(field.type)) for field in writer.schema
-            }  # TODO have nested conversion from Arrow to Python
-        )
+        features = {}
+
+        def parse_schema(schema, schema_dict):
+            for field in schema:
+                if pa.types.is_struct(field.type):
+                    schema_dict[field.name] = {}
+                    parse_schema(field.type, schema_dict[field.name])
+                elif pa.types.is_list(field.type) and pa.types.is_struct(field.type.value_type):
+                    schema_dict[field.name] = {}
+                    parse_schema(field.type.value_type, schema_dict[field.name])
+                else:
+                    schema_dict[field.name] = Value(str(field.type))
+
+        parse_schema(writer.schema, features)
+        self.info.features = Features(features)
+
+
+class MissingBeamOptions(ValueError):
+    pass
 
 
 class BeamBasedBuilder(DatasetBuilder):
     """Beam based Builder."""
 
+    # BeamBasedBuilder does not have dummy data for tests yet
+    test_dummy_data = False
+
     def __init__(self, *args, **kwargs):
+        self._beam_runner = kwargs.pop("beam_runner", None)
+        self._beam_options = kwargs.pop("beam_options", None)
         super(BeamBasedBuilder, self).__init__(*args, **kwargs)
-        self._beam_runner = kwargs.get("beam_runner")
-        self._beam_options = kwargs.get("beam_options")
         self._beam_writers = {}  # {split: beam_writer} mapping.
 
     def _make_split_generators_kwargs(self, prepare_split_kwargs):
@@ -694,39 +801,27 @@ class BeamBasedBuilder(DatasetBuilder):
         """
         raise NotImplementedError()
 
-    def _as_dataset(self, split=splits_lib.Split.TRAIN):
-        """Constructs a `Dataset`.
-
-        This is the internal implementation to overwrite called when user calls
-        `as_dataset`. It should read the pre-processed datasets files and generate
-        the `Dataset` object.
-
-        Args:
-            split: `nlp.Split` which subset of the data to read.
-
-        Returns:
-            `Dataset`
-        """
-
-        ds = ParquetReader(self._cache_dir, self.info).read(
-            name=self.name, instructions=split, split_infos=self.info.splits.values(),
-        )
-        return ds
-
-    def _download_and_prepare(self, dl_manager):
+    def _download_and_prepare(self, dl_manager, verify_infos):
         # Create the Beam pipeline and forward it to _prepare_split
         import apache_beam as beam
+        import nlp.utils.beam_utils as beam_utils
 
         beam_runner = self._beam_runner
         beam_options = self._beam_options
 
         if not beam_runner and not beam_options:
-            logger.warning(
+            usage_example = f"load_dataset('{self.name}', '{self.config.name}', beam_runner='DirectRunner')"
+            raise MissingBeamOptions(
                 "Trying to generate a dataset using Apache Beam, yet no Beam Runner "
-                "or PipelineOptions() has been provided. Please pass a "
-                "nlp.DownloadConfig(beam_runner=...) object to the "
-                "builder.download_and_prepare(download_config=...) method. "
-                "Default values will be used."
+                "or PipelineOptions() has been provided in `load_dataset` or in the "
+                "builder arguments. For big datasets it has to run on large-scale data "
+                "processing tools like Dataflow, Spark, etc. More information about "
+                "Apache Beam runners at "
+                "https://beam.apache.org/documentation/runners/capability-matrix/"
+                "\nIf you really want to run it locally because you feel like the "
+                "Dataset is small enough, you can use the local beam runner called "
+                "`DirectRunner` (you may run out of memory). \nExample of usage: "
+                "\n\t`{}`".format(usage_example)
             )
 
         beam_options = beam_options or beam.options.pipeline_options.PipelineOptions()
@@ -735,33 +830,44 @@ class BeamBasedBuilder(DatasetBuilder):
         # are better without it.
         beam_options.view_as(beam.options.pipeline_options.TypeOptions).pipeline_type_check = False
         # Use a single pipeline for all splits
-        with beam.Pipeline(runner=beam_runner, options=beam_options,) as pipeline:
-            super(BeamBasedBuilder, self)._download_and_prepare(
-                dl_manager, pipeline=pipeline,
-            )
-
+        pipeline = beam_utils.BeamPipeline(runner=beam_runner, options=beam_options,)
+        super(BeamBasedBuilder, self)._download_and_prepare(
+            dl_manager, verify_infos=False, pipeline=pipeline,
+        )  # TODO handle verify_infos in beam datasets
+        # Run pipeline
+        pipeline_results = pipeline.run()
+        pipeline_results.wait_until_finish()
+        metrics = pipeline_results.metrics()
         # Update `info.splits`.
         split_dict = self.info.splits
         for split_name, beam_writer in self._beam_writers.items():
-            num_examples, num_bytes = beam_writer.finalize()
+            m_filter = beam.metrics.MetricsFilter().with_namespace(namespace=split_name)
+            num_examples, num_bytes = beam_writer.finalize(metrics.query(m_filter))
             split_info = split_dict[split_name]
             split_info.num_examples = num_examples
             split_info.num_bytes = num_bytes
 
-    def _prepare_split(self, split_generator, pipeline):
+    def _save_info(self):
         import apache_beam as beam
 
-        os.makedirs(self._cache_dir, exist_ok=True)
+        fs = beam.io.filesystems.FileSystems
+        with fs.create(os.path.join(self._cache_dir, DATASET_INFO_FILENAME)) as f:
+            self.info._dump_info(f)
+        with fs.create(os.path.join(self._cache_dir, LICENSE_FILENAME)) as f:
+            self.info._dump_license(f)
+
+    def _prepare_split(self, split_generator, pipeline):
+        import apache_beam as beam
 
         split_name = split_generator.split_info.name
         output_prefix = filename_prefix_for_split(self.name, split_name)
         output_prefix = os.path.join(self._cache_dir, output_prefix)
 
         # To write examples to disk:
-        fname = "{}-{}.parquet".format(self.name, split_name)
+        fname = "{}-{}.arrow".format(self.name, split_name)
         fpath = os.path.join(self._cache_dir, fname)
         examples_type = self.info.features.type
-        beam_writer = BeamWriter(examples_type, path=fpath)
+        beam_writer = BeamWriter(examples_type, path=fpath, namespace=split_name, cache_dir=self._cache_dir)
         self._beam_writers[split_name] = beam_writer
 
         encode_example = self.info.features.encode_example
