@@ -17,6 +17,7 @@
 """Access datasets."""
 
 import importlib
+import inspect
 import json
 import logging
 import os
@@ -30,10 +31,10 @@ from urllib.parse import urlparse
 from filelock import FileLock
 
 from .arrow_dataset import Dataset
-from .builder import BuilderConfig, DatasetBuilder
+from .builder import DatasetBuilder
+from .info import DATASET_INFOS_DICT_FILE_NAME
 from .metric import Metric
 from .splits import Split
-from .utils.checksums_utils import CHECKSUMS_FILE_NAME, URLS_CHECKSUMS_FOLDER_NAME
 from .utils.download_manager import GenerateMode
 from .utils.file_utils import DownloadConfig, cached_path, hf_bucket_url
 
@@ -48,7 +49,10 @@ METRICS_MODULE = "nlp.metrics"
 
 
 def import_main_class(module_path, dataset=True):
-    """ Load the module """
+    """ Import a module at module_path and return its main class:
+        - a DatasetBuilder if dataset is True
+        - a Metric if dataset is False
+    """
     importlib.invalidate_caches()
     module = importlib.import_module(module_path)
 
@@ -61,6 +65,8 @@ def import_main_class(module_path, dataset=True):
     module_main_cls = None
     for name, obj in module.__dict__.items():
         if isinstance(obj, type) and issubclass(obj, main_cls_type):
+            if inspect.isabstract(obj):
+                continue
             module_main_cls = obj
             break
 
@@ -103,19 +109,33 @@ def files_to_hash(file_paths: List[str]):
 
 
 def convert_github_url(url_path: str):
+    """ Convert a link to a file on a github repo in a link to the raw github object.
+    """
     parsed = urlparse(url_path)
+    sub_directory = None
     if parsed.scheme in ("http", "https", "s3") and parsed.netloc == "github.com":
-        assert url_path.endswith(".py"), "External import from gitbhu should direcly point to a file ending with '.py'"
-        assert "blob" in url_path, "External import from gitbhu should direcly point to a file"
-        url_path = url_path.replace("blob", "raw")  # Point to the raw file
-    return url_path
+        if "blob" in url_path:
+            assert url_path.endswith(
+                ".py"
+            ), f"External import from github at {url_path} should point to a file ending with '.py'"
+            url_path = url_path.replace("blob", "raw")  # Point to the raw file
+        else:
+            # Parse github url to point to zip
+            github_path = parsed.path[1:]
+            repo_info, branch = github_path.split("/tree/") if "/tree/" in github_path else (github_path, "master")
+            repo_owner, repo_name = repo_info.split("/")
+            url_path = "https://github.com/{}/{}/archive/{}.zip".format(repo_owner, repo_name, branch)
+            sub_directory = f"{repo_name}-{branch}"
+    return url_path, sub_directory
 
 
 def get_imports(file_path: str):
-    """ Find whether we should import or clone additional files for a given processing script.
+    r"""
+        Find whether we should import or clone additional files for a given processing script.
         And list the import.
 
         We allow:
+        - library dependencies,
         - local dependencies and
         - external dependencies whose url is specified with a comment starting from "# From:' followed by the raw url to a file, an archive or a github repository.
             external dependencies will be downloaded (and extracted if needed in the dataset folder).
@@ -125,6 +145,7 @@ def get_imports(file_path: str):
         We don't recursively explore the additional import to download further files.
 
         ```python
+        import tensorflow
         import .c4_utils
         import .clicr.dataset-code.build_json_dataset  # From: https://raw.githubusercontent.com/clips/clicr/master/dataset-code/build_json_dataset
         ```
@@ -147,33 +168,51 @@ def get_imports(file_path: str):
                 continue
         if match.group(1):
             # The import starts with a '.', we will download the relevant file
+            if any(imp[1] == match.group(2) for imp in imports):
+                # We already have this import
+                continue
             if match.group(3):
                 # The import has a comment with 'From:', we'll retreive it from the given url
                 url_path = match.group(3)
-                url_path = convert_github_url(url_path)
-                imports.append(("external", match.group(2), url_path))
+                url_path, sub_directory = convert_github_url(url_path)
+                imports.append(("external", match.group(2), url_path, sub_directory))
             elif match.group(2):
                 # The import should be at the same place as the file
-                imports.append(("internal", match.group(2), match.group(2)))
+                imports.append(("internal", match.group(2), match.group(2), None))
         else:
-            imports.append(("library", match.group(2), match.group(2)))
+            imports.append(("library", match.group(2), match.group(2), None))
 
     return imports
 
 
-def prepare_module(path: str, download_config=None, dataset=True, **download_kwargs,) -> DatasetBuilder:
+def prepare_module(
+    path: str,
+    download_config: Optional[DownloadConfig] = None,
+    dataset: bool = True,
+    force_local_path: Optional[str] = None,
+    **download_kwargs,
+) -> DatasetBuilder:
     r"""
         Download/extract/cache a dataset (if dataset==True) or a metric (if dataset==False)
-        to add to the lib from a path or url which can be:
-            - a path to a local directory containing the dataset processing python script
-            - an url to a S3 directory with a dataset processing python script
 
         Dataset and metrics codes are cached inside the lib to allow easy import (avoid ugly sys.path tweaks)
         and using cloudpickle (among other things).
 
-        Return: tuple of
-            the unique id associated to the dataset/metric
-            the local path to the dataset/metric
+        Args:
+
+            path (str): path to the dataset or metric script, can be either:
+                - a path to a local directory containing the dataset processing python script
+                - an url to a S3 directory with a dataset processing python script
+            download_config (Optional ``nlp.DownloadConfig``: specific download configuration parameters.
+            dataset (bool): True if the script to load is a dataset, False if the script is a metric.
+            force_local_path (Optional str): Optional path to a local path to download and prepare the script to.
+                Used to inspect or modify the script folder.
+            **download_kwargs: optional attributes for DownloadConfig() which will override the attributes in download_config if supplied.
+
+        Return: ``str`` with
+
+            - the import path of the dataset/metric package if force_local_path is False: e.g. 'nlp.datasets.squad'
+            - the local path to the dataset/metric file if force_local_path is True: e.g. '/User/huggingface/nlp/datasets/squad/squad.py'
     """
     if download_config is None:
         download_config = DownloadConfig(**download_kwargs)
@@ -201,24 +240,24 @@ def prepare_module(path: str, download_config=None, dataset=True, **download_kwa
         file_path = hf_bucket_url(path, filename=name, dataset=dataset)
 
     base_path = os.path.dirname(file_path)  # remove the filename
-    checksums_file = os.path.join(base_path, URLS_CHECKSUMS_FOLDER_NAME, CHECKSUMS_FILE_NAME)
+    dataset_infos = os.path.join(base_path, DATASET_INFOS_DICT_FILE_NAME)
 
     # Load the module in two steps:
     # 1. get the processing file on the local filesystem if it's not there (download to cache dir)
     # 2. copy from the local file system inside the library to import it
     local_path = cached_path(file_path, download_config=download_config)
 
-    # Download the checksums file if available
+    # Download the dataset infos file if available
     try:
-        local_checksums_file_path = cached_path(checksums_file, download_config=download_config,)
+        local_dataset_infos_path = cached_path(dataset_infos, download_config=download_config,)
     except (FileNotFoundError, ConnectionError):
-        local_checksums_file_path = None
+        local_dataset_infos_path = None
 
     # Download external imports if needed
     imports = get_imports(local_path)
     local_imports = []
     library_imports = []
-    for import_type, import_name, import_path in imports:
+    for import_type, import_name, import_path, sub_directory in imports:
         if import_type == "library":
             library_imports.append(import_name)  # Import from a library
             continue
@@ -238,6 +277,8 @@ def prepare_module(path: str, download_config=None, dataset=True, **download_kwa
             raise ValueError("Wrong import_type")
 
         local_import_path = cached_path(url_or_filename, download_config=download_config,)
+        if sub_directory is not None:
+            local_import_path = os.path.join(local_import_path, sub_directory)
         local_imports.append((import_name, local_import_path))
 
     # Check library imports
@@ -257,11 +298,16 @@ def prepare_module(path: str, download_config=None, dataset=True, **download_kwa
     # path is: ./datasets|metrics/dataset|metric_name/hash_from_code/script.py
     # we use a hash to be able to have multiple versions of a dataset/metric processing file together
     hash = files_to_hash([local_path] + [loc[1] for loc in local_imports])
-    main_folder_path = os.path.join(DATASETS_PATH if dataset else METRICS_PATH, short_name)
-    hash_folder_path = os.path.join(main_folder_path, hash)
+
+    if force_local_path is None:
+        main_folder_path = os.path.join(DATASETS_PATH if dataset else METRICS_PATH, short_name)
+        hash_folder_path = os.path.join(main_folder_path, hash)
+    else:
+        main_folder_path = force_local_path
+        hash_folder_path = force_local_path
+
     local_file_path = os.path.join(hash_folder_path, name)
-    urls_checksums_dir = os.path.join(hash_folder_path, URLS_CHECKSUMS_FOLDER_NAME)
-    checksums_file_path = os.path.join(urls_checksums_dir, CHECKSUMS_FILE_NAME)
+    dataset_infos_path = os.path.join(hash_folder_path, DATASET_INFOS_DICT_FILE_NAME)
 
     # Prevent parallel disk operations
     lock_path = local_path + ".lock"
@@ -299,16 +345,15 @@ def prepare_module(path: str, download_config=None, dataset=True, **download_kwa
         else:
             logger.info("Found script file from %s to %s", file_path, local_file_path)
 
-        # Copy checksums file if needed
-        os.makedirs(urls_checksums_dir, exist_ok=True)
-        if not os.path.exists(checksums_file_path):
-            if local_checksums_file_path is not None:
-                logger.info("Copying checksums file from %s to %s", checksums_file, checksums_file_path)
-                shutil.copyfile(local_checksums_file_path, checksums_file_path)
+        # Copy dataset infos file if needed
+        if not os.path.exists(dataset_infos_path):
+            if local_dataset_infos_path is not None:
+                logger.info("Copying dataset infos file from %s to %s", dataset_infos, dataset_infos_path)
+                shutil.copyfile(local_dataset_infos_path, dataset_infos_path)
             else:
-                logger.info("Couldn't find checksums file at %s", checksums_file)
+                logger.info("Couldn't find dataset infos file at %s", dataset_infos)
         else:
-            logger.info("Found checksums file from %s to %s", checksums_file, checksums_file_path)
+            logger.info("Found dataset infos file from %s to %s", dataset_infos, dataset_infos_path)
 
         # Record metadata associating original dataset path with local unique folder
         meta_path = local_file_path.split(".py")[0] + ".json"
@@ -323,14 +368,28 @@ def prepare_module(path: str, download_config=None, dataset=True, **download_kwa
 
         # Copy all the additional imports
         for import_name, import_path in local_imports:
-            full_path_local_import = os.path.join(hash_folder_path, import_name + ".py")
-            if not os.path.exists(full_path_local_import):
-                logger.info("Copying local import from %s at %s", import_path, full_path_local_import)
-                shutil.copyfile(import_path, full_path_local_import)
+            if os.path.isfile(import_path):
+                full_path_local_import = os.path.join(hash_folder_path, import_name + ".py")
+                if not os.path.exists(full_path_local_import):
+                    logger.info("Copying local import file from %s at %s", import_path, full_path_local_import)
+                    shutil.copyfile(import_path, full_path_local_import)
+                else:
+                    logger.info("Found local import file from %s at %s", import_path, full_path_local_import)
+            elif os.path.isdir(import_path):
+                full_path_local_import = os.path.join(hash_folder_path, import_name)
+                if not os.path.exists(full_path_local_import):
+                    logger.info("Copying local import directory from %s at %s", import_path, full_path_local_import)
+                    shutil.copytree(import_path, full_path_local_import)
+                else:
+                    logger.info("Found local import directory from %s at %s", import_path, full_path_local_import)
             else:
-                logger.info("Found local import from %s at %s", import_path, full_path_local_import)
+                raise OSError(f"Error with local import at {import_path}")
+
+    if force_local_path is None:
+        module_path = ".".join([DATASETS_MODULE if dataset else METRICS_MODULE, short_name, hash, short_name])
+    else:
+        module_path = local_file_path
 
-    module_path = ".".join([DATASETS_MODULE if dataset else METRICS_MODULE, short_name, hash, short_name])
     return module_path
 
 
@@ -345,29 +404,26 @@ def load_metric(
     download_config: Optional[DownloadConfig] = None,
     **metric_init_kwargs,
 ) -> Metric:
-    """Fetches a `nlp.Metric` by string name.
+    r"""
+        Load a `nlp.Metric`.
 
-    Args:
-        name: `str`, the name of the `Metric` (the snake case
-            version of the class name). This can be either `"dataset_name"` or
-            `"dataset_name/config_name"` for datasets with `BuilderConfig`s.
-            As a convenience, this string may contain comma-separated keyword
-            arguments for the builder. For example `"foo_bar/a=True,b=3"` would use
-            the `FooBar` dataset passing the keyword arguments `a=True` and `b=3`
-            (for builders with configs, it would be `"foo_bar/zoo/a=True,b=3"` to
-            use the `"zoo"` config and pass to the builder keyword arguments `a=True`
-            and `b=3`).
-        **builder_init_kwargs: `dict` of keyword arguments passed to the
-            `DatasetBuilder`. These will override keyword arguments passed in `name`,
-            if any.
+        Args:
 
-    Returns:
-        A `nlp.Metric`.
+            path (``str``): path to the dataset processing script with the dataset builder. Can be either:
+                - a local path to processing script or the directory containing the script (if the script has the same name as the directory),
+                    e.g. ``'./dataset/squad'`` or ``'./dataset/squad/squad.py'``
+                - a dataset identifier on HuggingFace AWS bucket (list all available datasets and ids with ``nlp.list_datasets()``)
+                    e.g. ``'squad'``, ``'glue'`` or ``'openai/webtext'``
+            name (Optional ``str``): defining the name of the dataset configuration
+            process_id (Optional ``int``): for distributed evaluation: id of the process
+            num_process (Optional ``int``): for distributed evaluation: total number of processes
+            data_dir (Optional str): path to store the temporary predictions and references (default to `~/.nlp/`)
+            experiment_id (Optional str): An optional unique id for the experiment.
+            in_memory (bool): Weither to store the temporary results in memory (default: False)
+            download_config (Optional ``nlp.DownloadConfig``: specific download configuration parameters.
 
-    Raises:
-        MetricNotFoundError: if `name` is unrecognized.
+        Returns: `nlp.Metric`.
     """
-    # Download/copy metric script
     module_path = prepare_module(path, download_config=download_config, dataset=False)
     metric_cls = import_main_class(module_path, dataset=False)
     metric = metric_cls(
@@ -384,92 +440,84 @@ def load_metric(
 
 def load_dataset(
     path: str,
-    split: Optional[Union[str, Split]] = None,
     name: Optional[str] = None,
     version: Optional[str] = None,
-    description: Optional[str] = None,
     data_dir: Optional[str] = None,
     data_files: Union[Dict, List] = None,
-    dataset_config: Optional[BuilderConfig] = None,
+    split: Optional[Union[str, Split]] = None,
+    cache_dir: Optional[str] = None,
     download_config: Optional[DownloadConfig] = None,
     download_mode: Optional[GenerateMode] = None,
-    ignore_checksums: bool = False,
-    save_checksums: bool = False,
+    ignore_verifications: bool = False,
+    save_infos: bool = False,
     **config_kwargs,
-) -> Dataset:
-    """ Load a dataset.
+) -> Union[Dict[Split, Dataset], Dataset]:
+    r"""Load a dataset
 
-    This method does the following under the hood:
-        1. Download and import in the library the dataset loading script from ``path`` if it's not already cached inside the library.
+        This method does the following under the hood:
 
-            Processing scripts are small python scripts that define the citation, info and format of the dataset,
-            contain the URL to the original data files and the code to load examples from the original data files.
+            1. Download and import in the library the dataset loading script from ``path`` if it's not already cached inside the library.
 
-            You can find some of the scripts here: https://github.com/huggingface/nlp/datasets
-            and easily upload yours to share them using the CLI ``nlp-cli``.
+                Processing scripts are small python scripts that define the citation, info and format of the dataset,
+                contain the URL to the original data files and the code to load examples from the original data files.
 
-        2. Run the dataset loading script which will:
+                You can find some of the scripts here: https://github.com/huggingface/nlp/datasets
+                and easily upload yours to share them using the CLI ``nlp-cli``.
 
-            * Download the dataset file from the original URL (see the script) if it's not already downloaded and cached.
-            * Process and cache the dataset in typed Arrow tables for caching.
+            2. Run the dataset loading script which will:
 
-                Arrow table are arbitrarly long, typed tables which can store nested objects and be mapped to numpy/pandas/python standard types.
-                They can be directly access from drive, loaded in RAM or even streamed over the web.
+                * Download the dataset file from the original URL (see the script) if it's not already downloaded and cached.
+                * Process and cache the dataset in typed Arrow tables for caching.
 
-        3. Return a dataset build from the requested splits in ``split`` (default: all).
+                    Arrow table are arbitrarly long, typed tables which can store nested objects and be mapped to numpy/pandas/python standard types.
+                    They can be directly access from drive, loaded in RAM or even streamed over the web.
 
-    Args:
-        path: `str`, the name of the `Dataset` (the snake case
-            version of the class name). This can be either `"dataset_name"` or
-            `"dataset_name/config_name"` for datasets with `BuilderConfig`s.
-            As a convenience, this string may contain comma-separated keyword
-            arguments for the builder. For example `"foo_bar/a=True,b=3"` would use
-            the `FooBar` dataset passing the keyword arguments `a=True` and `b=3`
-            (for builders with configs, it would be `"foo_bar/zoo/a=True,b=3"` to
-            use the `"zoo"` config and pass to the builder keyword arguments `a=True`
-            and `b=3`).
-        split: `nlp.Split` or `str`, which split of the data to load. If None,
-            will return a `dict` with all splits (typically `nlp.Split.TRAIN` and
-            `nlp.Split.TEST`).
-        cache_dir: `str` (optional), directory to read/write data.
-            Defaults to "~/nlp".
-        in_memory: `bool`, if `True`, loads the dataset in memory which
-            increases iteration speeds. Note that if `True` and the dataset has
-            unknown dimensions, the features will be padded to the maximum
-            size across the dataset.
-        builder_kwargs: `dict` (optional), keyword arguments to be passed to the
-            `nlp.DatasetBuilder` constructor. `data_dir` will be passed
-            through by default.
+            3. Return a dataset build from the requested splits in ``split`` (default: all).
 
-    Returns:
-        ds: `tf.data.Dataset`, the dataset requested, or if `split` is None, a
-            `dict<key: nlp.Split, value: nlp.data.Dataset>`. If `batch_size=-1`,
-            these will be full datasets as `tf.Tensor`s.
+        Args:
+
+            path (``str``): path to the dataset processing script with the dataset builder. Can be either:
+                - a local path to processing script or the directory containing the script (if the script has the same name as the directory),
+                    e.g. ``'./dataset/squad'`` or ``'./dataset/squad/squad.py'``
+                - a datatset identifier on HuggingFace AWS bucket (list all available datasets and ids with ``nlp.list_datasets()``)
+                    e.g. ``'squad'``, ``'glue'`` or ``'openai/webtext'``
+            name (Optional ``str``): defining the name of the dataset configuration
+            version (Optional ``str``): defining the version of the dataset configuration
+            data_files (Optional ``str``): defining the data_files of the dataset configuration
+            data_dir (Optional ``str``): defining the data_dir of the dataset configuration
+            split (`nlp.Split` or `str`): which split of the data to load.
+                If None, will return a `dict` with all splits (typically `nlp.Split.TRAIN` and `nlp.Split.TEST`).
+                If given, will return a single Dataset.
+                Splits can be combined and specified like in tensorflow-datasets.
+            cache_dir (Optional ``str``): directory to read/write data. Defaults to "~/nlp".
+            download_config (Optional ``nlp.DownloadConfig``: specific download configuration parameters.
+            download_mode (Optional `nlp.GenerateMode`): select the download/generate mode - Default to REUSE_DATASET_IF_EXISTS
+            ignore_verifications (bool): Ignore the verifications of the downloaded/processed dataset information (checksums/size/splits/...)
+            save_infos (bool): Save the dataset information (checksums/size/splits/...)
+            **config_kwargs (Optional ``dict``): keyword arguments to be passed to the ``nlp.BuilderConfig`` and used in the ``nlp.DatasetBuilder``.
+
+        Returns: ``nlp.Dataset`` or ``Dict[nlp.Split, nlp.Dataset]``
+
+            if `split` is not None: the dataset requested,
+            if `split` is None, a `dict<key: nlp.Split, value: nlp.Dataset>` with each split.
     """
-
-    # Download/copy dataset script
+    # Download/copy dataset processing script
     module_path = prepare_module(path, download_config=download_config, dataset=True)
 
-    # Get dataset builder class
+    # Get dataset builder class from the processing script
     builder_cls = import_main_class(module_path, dataset=True)
 
-    # Instantiate dataset builder
+    # Instantiate the dataset builder
     builder_instance = builder_cls(
-        config=dataset_config,
-        name=name,
-        version=version,
-        description=description,
-        data_dir=data_dir,
-        data_files=data_files,
-        **config_kwargs,
+        cache_dir=cache_dir, name=name, version=version, data_dir=data_dir, data_files=data_files, **config_kwargs,
     )
 
     # Download and prepare data
     builder_instance.download_and_prepare(
         download_config=download_config,
         download_mode=download_mode,
-        ignore_checksums=ignore_checksums,
-        save_checksums=save_checksums,
+        ignore_verifications=ignore_verifications,
+        save_infos=save_infos,
     )
 
     # Build dataset for splits
